/*
    pcdump -- dump PCMCIA configuration information
*/

#include <stdio.h>
#include <sys/types.h>
#include <sys/fcntl.h>
#include <linux/pcmcia.h>

#define check(s, a) if ((a) < 0) { perror(s); exit(1); } else

void dump_card(char *name)
  {
  int i, fd, val;
  struct pcmcia_io_map io;
  struct pcmcia_mem_map mem;

  printf("Opening %s...\n", name);
  check("open()", fd = open(name, O_RDONLY))
  check("ioctl()", ioctl(fd, PCMCIA_GET_STATUS, &val));
  printf("  Card status: ");
  printf(val & PC_DETECT ? "present" : "vacant");
  printf(val & PC_POWERON ? ", power on" : ", power off");
  printf(val & PC_READY ? ", ready" : ", busy");
  if (val & PC_STSCHG) printf(", status change");
  if (val & PC_WRPROT) printf(", write protect");
  if (val & PC_BATDEAD)
    printf(", battery dead");
  else
    if (val & PC_BATWARN) printf(", battery warning");
  printf("\n");
  
  check("ioctl()", ioctl(fd, PCMCIA_GET_FLAGS, &val));
  printf("  Configuration: ");
  printf(val & PC_IOCARD ? "I/O card" : "memory card");
  printf(val & PC_RESET ? ", reset active\n" : "\n");
  printf("  Power: ");
  if (val & PC_PWR_AUTO) printf("auto select, ");
  printf("Vcc is ");
  switch (val & PC_VCC_MASK) {
    case PC_PWR_OFF: printf("off"); break;
    case PC_VCC_3V: printf("3v"); break;
    case PC_VCC_5V: printf("5v"); break;
    }
  printf(", Vpp is ");
  switch (val & PC_VPP_MASK) {
    case PC_PWR_OFF: printf("off"); break;
    case PC_VPP_5V: printf("5v"); break;
    case PC_VPP_12V: printf("12v"); break;
    }
  printf("\n");

  check("ioctl()", ioctl(fd, PCMCIA_GET_IRQ, &val));
  printf("  I/O interrupt = %d\n", val);
  
  printf("  I/O windows:\n");
  for (i = 0; i < 2; i++) {
    io.map = i;
    check("ioctl()", ioctl(fd, PCMCIA_GET_IO_MAP, &io));
    if (io.flags & MAP_ACTIVE) {
      printf("    %d: ", i);
      printf(io.flags & MAP_16BIT ? "16 bit" : " 8 bit");
      printf(", 0x%04X -> 0x%04X", io.start, io.stop);
      printf("\n");
      }
    }

  printf("  Memory windows:\n");
  for (i = 0; i < 5; i++) {
    mem.map = i;
    check("ioctl()", ioctl(fd, PCMCIA_GET_MEM_MAP, &mem));
    if (mem.flags & MAP_ACTIVE) {
      printf("    %d: ", i);
      printf(mem.flags & MAP_16BIT ? "16 bit" : " 8 bit");
      printf(", 0x%05X -> 0x%05X at 0x%05X", mem.card_start,
	     mem.card_start+mem.sys_stop-mem.sys_start, mem.sys_start);
      printf("\n");
      }
    }
  }
  
void main(int argc, char *argv[])
  {
  int i;

  if (argc < 2) {
    fprintf(stderr, "usage: %s [dev] ...\n", argv[0]);
    exit(1);
    }
  for (i = 1; i < argc; i++)
    dump_card(argv[i]);
  }
  
