#ifndef _LINUX_PCMCIA_H
#define _LINUX_PCMCIA_H

#include <sys/ioctl.h>

#ifdef __KERNEL__

struct pcmcia_slot {
  u_int flags;
  int event;
  struct wait_queue *queue;
  };

#define SLOT_BUSY   0x01
#define SLOT_EVENT  0x02

struct pcmcia_ctrl {
  int csc_irq;
  void (*csc_on)(u_short slot, int irq);
  void (*csc_off)(u_short slot);
  int (*csc_poll)(u_short slot);
  int (*ioctl)(u_short slot, u_int cmd, u_long arg);
  };

#endif /* KERNEL */

/* Configuration registers found in attribute memory */
#define PCMCIA_COR	0x00	/* Configuration and Option Register */
#define PCMCIA_CCSR	0x02	/* Card Configuration and Status Register */
#define PCMCIA_PIR	0x04	/* Pin Replacement Register */
#define PCMCIA_SCR	0x06	/* Socket and Copy Register */

/* Now register bits, ordered by reg # */

/* For Configuration and Option Register (PCMCIA_COR) */
#define PCMCIA_MEMIO	0x01	/* Use I/O Space */
#define PCMCIA_CNFGIO	0x0e	/* I/O decoding configuration */
#define PCMCIA_LVLREQ	0x40	/* Generate level mode interrupts */
#define PCMCIA_SRESET	0x80	/* Reset Card */

/* For Card Configuration and Status Register (PCMCIA_CCSR) */
#define PCMCIA_INTR	0x02	/* Interrupt Pending */
#define PCMCIA_PWRDWN	0x04	/* Power down */
#define PCMCIA_AUDIO	0x08 	/* Audio -- enable speaker */
#define PCMCIA_IOIS8	0x20	/* IOis8 */
#define PCMCIA_SIGCHG	0x40	/* SigChg -- enable status change */
#define PCMCIA_CHANGED	0x80	/* Changed */

/* For Socket and Copy Register (PCMCIA_SCR) */
#define PCMCIA_SOCKNUM	0x0f	/* Which socket I'm sitting in */
#define PCMCIA_COPNUM	0x70	/* Which instance I am. */

/* Definitions for card status flags */
#define PC_DETECT	0x01
#define PC_BATDEAD	0x02
#define PC_BATWARN	0x04
#define PC_WRPROT	0x08
#define PC_READY	0x10
#define PC_POWERON	0x20
#define PC_GPI		0x40
#define PC_STSCHG	0x80

/* Various card configuration flags */
#define PC_PWR_OFF	0x00
#define PC_VCC_MASK	0x03
#define PC_VCC_3V	0x01
#define PC_VCC_5V	0x02
#define PC_VPP_MASK	0x0C
#define PC_VPP_5V	0x04
#define PC_VPP_12V	0x08
#define PC_PWR_AUTO	0x10
#define PC_IOCARD	0x20
#define PC_RESET	0x40

/* Flags for I/O port and memory windows */
#define MAP_ACTIVE	0x01
#define MAP_16BIT	0x02
#define MAP_WAIT	0x04
#define MAP_0WS		0x08
#define MAP_CS16	0x10
#define MAP_WRPROT	0x20
#define MAP_ATTRIB	0x40

/* Definitions for I/O port window ioctl calls */
struct pcmcia_io_map {
  u_char map, flags;
  u_short start, stop;
  };

/* Definitions for memory window ioctl calls */
struct pcmcia_mem_map {
  u_char map, flags;
  u_char extra_ws;
  u_long sys_start, sys_stop;
  u_long card_start;
  };

/* XXX The PCMCIA Standards probably specify how large this can be */
struct pcmcia_attr_mem {
     u_short buf[1024];
};

/* Definitions for PCMCIA register ioctl calls */
struct pcmcia_register {
     u_char reg, val;
     u_long ofs;
};

/* ioctl definitions for PCMCIA devices */
#define PCMCIA_GET_STATUS	_IOR ('p', 0, u_int)
#define PCMCIA_GET_FLAGS	_IOR ('p', 1, u_int)
#define PCMCIA_SET_FLAGS	_IOW ('p', 2, u_int)
#define PCMCIA_GET_IRQ		_IOR ('p', 3, u_int)
#define PCMCIA_SET_IRQ		_IOW ('p', 4, u_int)
#define PCMCIA_GET_IO_MAP	_IOWR('p', 5, struct pcmcia_io_map)
#define PCMCIA_SET_IO_MAP	_IOW ('p', 6, struct pcmcia_io_map)
#define PCMCIA_GET_MEM_MAP	_IOWR('p', 7, struct pcmcia_mem_map)
#define PCMCIA_SET_MEM_MAP	_IOW ('p', 8, struct pcmcia_mem_map)
#define PCMCIA_GET_ATTR_MEM	_IOR ('p', 9, struct pcmcia_attr_mem)
#define PCMCIA_GET_REG		_IOR ('p', 10, struct pcmcia_register)
#define PCMCIA_SET_REG		_IOW ('p', 11, struct pcmcia_register)

#endif /* _LINUX_PCMCIA_H */
